import os
import sys
import json
import random
import pandas as pd
from .notify import LINENotifyBot
from .utils import *


def generate_topic(data):
	idx = random.randint(0, len(data) - 1)
	topic = {'Topic': data.loc[idx]['Topic'], 
			'Notes': data.loc[idx]['Notes'], 
			'A': data.loc[idx]['A'], 
			'B': data.loc[idx]['B'], 
			'C': data.loc[idx]['C'], 
			'D': data.loc[idx]['D'], 
			'E': data.loc[idx]['E'], 
			'F': data.loc[idx]['F'],
			'G': data.loc[idx]['G'],
			'H': data.loc[idx]['H']}
	print_topic(topic)
	return idx, topic


def generate_act():
	acts = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']
	for act in random.sample(acts, len(acts)):
		yield act


def start_round(topic, players, bots, act):
	for player in random.sample(players, len(players)):
		try:
			os.system('clear')
			action = next(act)
			print("Press enter to view " + player + "'s action")
			input()
			action_msg = action + ": " + topic[action]
			print(action_msg, end = "\n\n")
			if bots[player] is not None:
				bots[player].send(msg = get_topic_string(topic) + player + "のお題は↓\n" + action_msg)
			print("Press enter if you finish checking your action...")
			input()
		except StopIteration:
			break


def setup_bot():
	bots = {k: LINENotifyBot(token = v) if v != "" else None for k, v in setup().items()}
	return bots


def setup():
	if parse_name_option():
		players = {}
		input_str = input("Enter number of players: ")
		try: 
			n = int(input_str)
		except Exception as e:
			print(e)
			print("Number of players should be an integer.")
			raise e

		for i in range(n):
			name = input("Player " + str(i + 1) + ": ")
			while name == "":
				print("Name should not be empty.")
				name = input("Player " + str(i + 1) + ": ")
			token = input("LINE Notify token of player " + str(i + 1) + ": ")
			players[name] = token

		return players
	else:
		path = os.path.join(os.path.dirname(__file__), 'data', 'tokens.txt')
		if os.path.exists(path):
			with open(path, 'r') as f:
				data = f.read()
			return json.loads(data)
		else:
			raise Exception(f"Could not load existing tokens data from {path}. Use the --newname option to create new users or create a tokens.txt file.")


def main(path, bots):
	# Initialize player
	players = bots.keys()

	# Prepare topics
	topic_data = pd.read_csv(path)
	topic_data = topic_data.fillna({'Notes': "-"})

	# Welcoming
	os.system('clear')
	show_banner()
	show_players(players)
	print("Press enter to continue or enter q to quit...")
	sys.exit() if input() == 'q' else os.system('clear')

	# Main game loop
	while len(topic_data) > 0:
		# Generate new topic		
		os.system('clear')
		idx, topic = generate_topic(topic_data)
		topic_data.drop([idx])
		input("Press enter to view your actions...")

		# Check actions
		start_round(topic, players, bots, generate_act())
		os.system('clear')

		# Vote
		print_topic(topic)
		print("Start voting...", end = "\n\n")

		# Go to new topic
		if input("Press enter to go to the next topic or enter q to quit...") == 'q':
			sys.exit()

	print("No more topics left...")


if __name__ == '__main__':
	main('data/topics.csv', setup_bot())


import os

def init_tokens_settings():
	tokens_path = input("Enter path to tokens.txt: ")
	while not os.path.exists(tokens_path):
		print(f"Cannot find tokens file at {tokens_path}")
		tokens_path = input("Enter path to tokens.txt: ")

	with open(tokens_path, 'r') as f:
		tokens = f.read()

	tokens_settings_path = os.path.join(os.path.dirname(__file__), 'data', 'tokens.txt')
	with open(tokens_settings_path, 'w') as f:
		print(f"Writing to {tokens_settings_path}")
		print(tokens)
		f.write(tokens)

def init_topics_settings():
	topics_path = input("Enter path to topics.csv: ")
	while not os.path.exists(topics_path):
		print(f"Cannot find topics file at {topics_path}")
		topics_path = input("Enter path to topics.csv: ")

	with open(topics_path, 'r') as f:
		topics = f.read()

	topics_settings_path = os.path.join(os.path.dirname(__file__), 'data', 'topics.csv')
	with open(topics_settings_path, 'w') as f:
		print(f"Writing to {topics_settings_path}")
		print(topics)
		f.write(topics)


import argparse

def show_banner():
	banner = """
=================================================\n\n
                 はぁって言うゲーム\n\n
=================================================\n\n
	"""	
	print(banner)


def show_players(players):
	player_list = "Players: "
	for i, name in enumerate(players):
		player_list += name
		if not i == len(players) - 1:
			player_list += " / "
	player_list += "\n\n"

	print(player_list)


def parse_name_option():
	parser = argparse.ArgumentParser()
	parser.add_argument('-n', '--newname', action='store_true')
	return parser.parse_args().newname


def print_topic(topic):
	print("====================================", end="\n\n")
	print("Topic:", topic['Topic'], end="\n\n")
	print("Notes:", topic['Notes'], end="\n\n")
	print("Act A:", topic['A'], end="\n\n")
	print("Act B:", topic['B'], end="\n\n")
	print("Act C:", topic['C'], end="\n\n")
	print("Act D:", topic['D'], end="\n\n")
	print("Act E:", topic['E'], end="\n\n")
	print("Act F:", topic['F'], end="\n\n")
	print("Act G:", topic['G'], end="\n\n")
	print("Act H:", topic['H'], end="\n\n")
	print("====================================", end="\n\n")


def get_topic_string(topic):
	return f"""\n
=======================\n
テーマ: {topic['Topic']}\n
指示: {topic['Notes']}\n
A: {topic['A']}\n
B: {topic['B']}\n
C: {topic['C']}\n
D: {topic['D']}\n
E: {topic['E']}\n
F: {topic['F']}\n
G: {topic['G']}\n
H: {topic['H']}\n
=======================\n\n
"""


import os
from .main import main, setup_bot
from .settings import init_tokens_settings, init_topics_settings


tokens_path = os.path.join(os.path.dirname(__file__), 'data', 'tokens.txt')
topics_path = os.path.join(os.path.dirname(__file__), 'data', 'topics.csv')


if not os.path.exists(tokens_path):
	print("Tokens settings file is missing. Initializing settings...")	
	init_tokens_settings()

elif input("Update tokens? [y/n]  ") == 'y':
	init_tokens_settings()


if not os.path.exists(topics_path):
	print("Topics settings file is missing. Initializing settings...")	
	init_topics_settings()

elif input("Update topics? [y/n]  ") == 'y':
	init_topics_settings()


if os.path.exists(topics_path):
	with open(topics_path, 'r') as f:
		line_count = 0
		for line in f:
			if line != '\n':
				line_count += 1
	if line_count < 3:
		print("Need at least 3 topics to start. Initializing settings...")
		init_topics_settings()


main(topics_path, setup_bot())


import requests

class LINENotifyBot:

	API_URL = "https://notify-api.line.me/api/notify"

	def __init__(self, token):
		self.headers = {'Authorization': 'Bearer ' + token}

	def send(self, msg):
		payload = {'message': msg}
		r = requests.post(self.API_URL, headers=self.headers, params=payload)


import setuptools

with open('README.md', 'r', encoding = 'utf-8') as fh:
	long_description = fh.read()

setuptools.setup(
	name = "hateyugemu",
	version = "0.0.1",
	author = "Kai Wang",
	description = "A board game.",
	long_description = long_description,
	long_description_content_type = "text/markdown",
	url = "https://github.com/kaiwinut/hateyugemu",
	project_urls = {
		"Bug Reports": "https://github.com/kaiwinut/hateyugemu/issues",
		"Source Code": "https://github.com/kaiwinut/hateyugemu",
	},
	classifiers = [
		"Programming Language :: Python :: 3",
		"Programming Language :: Python :: 3 :: Only",
		"Programming Language :: Python :: 3.7",
		"Programming Language :: Python :: 3.8",
		"Programming Language :: Python :: 3.9",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	],
	packages = setuptools.find_packages(where = "src"),
	package_dir = {"": "src"},
	include_package_data = True,
	package_data = {'hateyugemu': ['data/*.txt', 'data/*.csv']},
	python_requires = ">=3.7",
)